@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import logging
+from enum import Enum
 from typing import List, Optional
 
 import numpy as np
@@ -10,11 +11,22 @@ from plotly.subplots import make_subplots
 from zvt.contract.api import decode_entity_id
 from zvt.contract.data_type import Bean
 from zvt.contract.normal_data import NormalData
+from zvt.utils import to_string
 from zvt.utils.pd_utils import pd_is_not_null
 
 logger = logging.getLogger(__name__)
 
 
+class ChartType(Enum):
+    kline = 'kline'
+    line = 'line'
+    area = 'area'
+    scatter = 'scatter'
+    histogram = 'histogram'
+    pie = 'pie'
+
+
+@to_string
 class Rect(Bean):
 
     def __init__(self, x0=None, y0=None, x1=None, y1=None) -> None:
@@ -28,8 +40,9 @@ class Rect(Bean):
 
 class Draw(object):
     def draw_kline(self, width=None, height=None, title=None, keep_ui_state=True, show=False, **kwargs):
-        return self._draw('kline', width=width, height=height, title=title, keep_ui_state=keep_ui_state, show=show,
-                          **kwargs)
+        return self.draw(ChartType.kline, width=width, height=height, title=title, keep_ui_state=keep_ui_state,
+                         show=show,
+                         **kwargs)
 
     def draw_line(self, width=None, height=None, title=None, keep_ui_state=True, show=False, **kwargs):
         return self.draw_scatter(mode='lines', width=width, height=height, title=title,
@@ -41,23 +54,33 @@ class Draw(object):
 
     def draw_scatter(self, mode='markers', width=None, height=None,
                      title=None, keep_ui_state=True, show=False, **kwargs):
-        return self._draw('scatter', mode=mode, width=width, height=height, title=title, keep_ui_state=keep_ui_state,
-                          show=show, **kwargs)
+        return self.draw(ChartType.scatter, mode=mode, width=width, height=height, title=title,
+                         keep_ui_state=keep_ui_state,
+                         show=show, **kwargs)
 
-    def _draw(self,
-              main_chart='kline',
-              sub_chart='bar',
-              mode='lines',
-              width=None,
-              height=None,
-              title=None,
-              keep_ui_state=True,
-              show=False,
-              **kwargs):
+    def draw_histogram(self, width=None, height=None, title=None, keep_ui_state=True, show=False, **kwargs):
+        return self.draw(ChartType.histogram, width=width, height=height, title=title, keep_ui_state=keep_ui_state,
+                         show=show, **kwargs)
+
+    def draw_pie(self, width=None, height=None, title=None, keep_ui_state=True, show=False, **kwargs):
+        return self.draw(ChartType.pie, width=width, height=height, title=title, keep_ui_state=keep_ui_state,
+                         show=show, **kwargs)
+
+    def draw(self,
+             main_chart=ChartType.kline,
+             sub_chart='bar',
+             mode='lines',
+             width=None,
+             height=None,
+             title=None,
+             keep_ui_state=True,
+             show=False,
+             **kwargs):
 
         raise NotImplementedError()
 
     def default_layout(self,
+                       main_chart=None,
                        width=None,
                        height=None,
                        title=None,
@@ -68,6 +91,49 @@ class Draw(object):
         else:
             uirevision = None
 
+        if main_chart == ChartType.histogram:
+            xaxis = None
+        else:
+            xaxis = dict(
+                linecolor="#BCCCDC",
+                showgrid=False,
+                showspikes=True,  # Show spike line for X-axis
+                # Format spike
+                spikethickness=2,
+                spikedash="dot",
+                spikecolor="#999999",
+                spikemode="across",
+                rangeselector=dict(
+                    buttons=list([
+                        dict(count=1,
+                             label="1m",
+                             step="month",
+                             stepmode="backward"),
+                        dict(count=3,
+                             label="3m",
+                             step="month",
+                             stepmode="backward"),
+                        dict(count=6,
+                             label="6m",
+                             step="month",
+                             stepmode="backward"),
+                        dict(count=1,
+                             label="YTD",
+                             step="year",
+                             stepmode="todate"),
+                        dict(count=1,
+                             label="1y",
+                             step="year",
+                             stepmode="backward"),
+                        dict(step="all")
+                    ])
+                ),
+                rangeslider=dict(
+                    visible=True,
+                ),
+                type="date"
+            )
+
         return dict(showlegend=True,
                     plot_bgcolor="#FFF",
                     hovermode="x",
@@ -84,45 +150,7 @@ class Draw(object):
                         linecolor="#BCCCDC",
                         showgrid=False,
                     ),
-                    xaxis=dict(
-                        linecolor="#BCCCDC",
-                        showgrid=False,
-                        showspikes=True,  # Show spike line for X-axis
-                        # Format spike
-                        spikethickness=2,
-                        spikedash="dot",
-                        spikecolor="#999999",
-                        spikemode="across",
-                        rangeselector=dict(
-                            buttons=list([
-                                dict(count=1,
-                                     label="1m",
-                                     step="month",
-                                     stepmode="backward"),
-                                dict(count=3,
-                                     label="3m",
-                                     step="month",
-                                     stepmode="backward"),
-                                dict(count=6,
-                                     label="6m",
-                                     step="month",
-                                     stepmode="backward"),
-                                dict(count=1,
-                                     label="YTD",
-                                     step="year",
-                                     stepmode="todate"),
-                                dict(count=1,
-                                     label="1y",
-                                     step="year",
-                                     stepmode="backward"),
-                                dict(step="all")
-                            ])
-                        ),
-                        rangeslider=dict(
-                            visible=True,
-                        ),
-                        type="date"
-                    ),
+                    xaxis=xaxis,
                     legend_orientation="h",
                     hoverlabel={"namelength": -1},
                     **layout_params)
@@ -142,11 +170,18 @@ class Drawable(object):
                         rects=self.drawer_rects())
         return drawer
 
-    def draw(self, main_chart='kline', width=None, height=None, title=None, keep_ui_state=True, show=False, **kwargs):
-        return self.drawer()._draw(main_chart=main_chart, width=width, height=height, title=title,
-                                   keep_ui_state=keep_ui_state,
-                                   show=show,
-                                   **kwargs)
+    def draw(self,
+             main_chart=ChartType.kline,
+             width=None,
+             height=None,
+             title=None,
+             keep_ui_state=True,
+             show=False,
+             **kwargs):
+        return self.drawer().draw(main_chart=main_chart, width=width, height=height, title=title,
+                                  keep_ui_state=keep_ui_state,
+                                  show=show,
+                                  **kwargs)
 
     def drawer_main_df(self) -> Optional[pd.DataFrame]:
         return None
@@ -201,16 +236,16 @@ class StackedDrawer(Draw):
                               domain=[domain_range[0] + part * (index - start_index),
                                       domain_range[0] + part * (index - start_index + 1)])
 
-    def _draw(self,
-              main_chart='kline',
-              sub_chart='bar',
-              mode='lines',
-              width=None,
-              height=None,
-              title=None,
-              keep_ui_state=True,
-              show=False,
-              **kwargs):
+    def draw(self,
+             main_chart=ChartType.kline,
+             sub_chart='bar',
+             mode='lines',
+             width=None,
+             height=None,
+             title=None,
+             keep_ui_state=True,
+             show=False,
+             **kwargs):
         stacked_fig = go.Figure()
 
         total = len(self.drawers)
@@ -256,10 +291,12 @@ class StackedDrawer(Draw):
                                           yref=y)
 
             # annotations
-            stacked_fig.layout['annotations'] = annotations(drawer.annotation_df, yref=y)
+            if pd_is_not_null(drawer.annotation_df):
+                stacked_fig.layout['annotations'] = annotations(drawer.annotation_df, yref=y)
 
         stacked_fig.update_layout(
-            self.default_layout(width=width, height=height, title=title, keep_ui_state=keep_ui_state))
+            self.default_layout(main_chart=main_chart, width=width, height=height, title=title,
+                                keep_ui_state=keep_ui_state))
 
         if show:
             stacked_fig.show()
@@ -341,7 +378,7 @@ class Drawer(Draw):
         return self.sub_data_list is not None and not self.sub_data_list[0].empty()
 
     def make_traces(self,
-                    main_chart='kline',
+                    main_chart=ChartType.kline,
                     sub_chart='bar',
                     mode='lines',
                     yaxis='y',
@@ -358,16 +395,43 @@ class Drawer(Draw):
                 pass
 
             # 构造主图
-            if main_chart == 'kline':
+            if main_chart == ChartType.kline:
                 trace_name = '{}_kdata'.format(code)
                 trace = go.Candlestick(x=df.index, open=df['open'], close=df['close'], low=df['low'], high=df['high'],
                                        name=trace_name, yaxis=yaxis, **kwargs)
                 traces.append(trace)
-            elif main_chart == 'scatter':
+            elif main_chart == ChartType.scatter:
                 for col in df.columns:
                     trace_name = '{}_{}'.format(code, col)
                     ydata = df[col].values.tolist()
                     traces.append(go.Scatter(x=df.index, y=ydata, mode=mode, name=trace_name, yaxis=yaxis, **kwargs))
+            elif main_chart == ChartType.histogram:
+                for col in df.columns:
+                    trace_name = '{}_{}'.format(code, col)
+                    x = df[col].tolist()
+                    trace = go.Histogram(
+                        x=x,
+                        name=trace_name,
+                        **kwargs
+                    )
+                    traces.append(trace)
+                    annotation = [dict(
+                        entity_id=entity_id,
+                        timestamp=x[-1],
+                        value=0,
+                        flag=f'{trace_name}:{x[-1]}',
+                    )]
+                    annotation_df = pd.DataFrame.from_records(annotation, index=['entity_id', 'timestamp'])
+                    if pd_is_not_null(self.annotation_df):
+                        self.annotation_df = pd.concat([self.annotation_df, annotation_df])
+                    else:
+                        self.annotation_df = annotation_df
+            elif main_chart == ChartType.pie:
+                for _, row in df.iterrows():
+                    traces.append(go.Pie(name=entity_id, labels=df.columns.tolist(), values=row.tolist(), **kwargs))
+                    break
+            else:
+                assert False
 
             # 构造主图指标
             if self.factor_data_list:
@@ -408,7 +472,7 @@ class Drawer(Draw):
                             if not the_sub_chart:
                                 the_sub_chart = sub_chart
 
-                            if the_sub_chart == 'line':
+                            if the_sub_chart == ChartType.line:
                                 sub_trace = go.Scatter(x=sub_df.index, y=ydata, name=trace_name, yaxis='y2',
                                                        marker=dict(color=colors))
                             else:
@@ -429,16 +493,16 @@ class Drawer(Draw):
                               )
             fig.update_shapes(dict(xref='x', yref=yaxis))
 
-    def _draw(self,
-              main_chart='kline',
-              sub_chart='bar',
-              mode='lines',
-              width=None,
-              height=None,
-              title=None,
-              keep_ui_state=True,
-              show=False,
-              **kwargs):
+    def draw(self,
+             main_chart=ChartType.kline,
+             sub_chart='bar',
+             mode='lines',
+             width=None,
+             height=None,
+             title=None,
+             keep_ui_state=True,
+             show=False,
+             **kwargs):
         yaxis = 'y'
         traces, sub_traces = self.make_traces(main_chart=main_chart, sub_chart=sub_chart, mode=mode, yaxis=yaxis,
                                               **kwargs)
@@ -454,13 +518,15 @@ class Drawer(Draw):
         # 绘制矩形
         self.add_rects(fig, yaxis=yaxis)
 
-        fig.update_layout(self.default_layout(width=width, height=height, title=title, keep_ui_state=keep_ui_state))
+        fig.update_layout(self.default_layout(main_chart=main_chart, width=width, height=height, title=title,
+                                              keep_ui_state=keep_ui_state))
 
         if sub_traces:
             fig.update_layout(xaxis_rangeslider_visible=False)
             fig.update_layout(xaxis2_rangeslider_visible=True, xaxis2_rangeslider_thickness=0.1)
         # 绘制标志
-        fig.layout['annotations'] = annotations(self.annotation_df, yref=yaxis)
+        if pd_is_not_null(self.annotation_df):
+            fig.layout['annotations'] = annotations(self.annotation_df, yref=yaxis)
 
         if show:
             fig.show()
@@ -534,11 +600,6 @@ def annotations(annotation_df: pd.DataFrame, yref='y'):
     return None
 
 
-def distribute(df, col):
-    import plotly.express as px
-    fig = px.histogram(df, x=col)
-    fig.show()
-
 if __name__ == '__main__':
     from zvt.factors.zen import ZenFactor
 
@@ -553,4 +614,4 @@ if __name__ == '__main__':
     #
     # Drawer(main_df=df, sub_df_list=[sub_df]).draw_kline(show=True)
 # the __all__ is generated
-__all__ = ['Rect', 'Draw', 'Drawable', 'StackedDrawer', 'Drawer', 'annotations', 'distribute']
\ No newline at end of file
+__all__ = ['Rect', 'Draw', 'Drawable', 'StackedDrawer', 'Drawer', 'annotations']
