@@ -78,9 +78,18 @@ class EntityEventRecorder(Recorder):
     entity_provider: str = None
     entity_schema: TradableEntity = None
 
-    def __init__(self, force_update=False, sleeping_time=10, exchanges=None, entity_ids=None, codes=None,
-                 day_data=False, entity_filters=None, ignore_failed=True) -> None:
+    def __init__(self,
+                 force_update=False,
+                 sleeping_time=10,
+                 exchanges=None,
+                 entity_ids=None,
+                 code=None,
+                 codes=None,
+                 day_data=False,
+                 entity_filters=None,
+                 ignore_failed=True) -> None:
         """
+        :param code:
         :param ignore_failed:
         :param entity_filters:
         :param exchanges:
@@ -97,7 +106,10 @@ class EntityEventRecorder(Recorder):
 
         # setup the entities you want to record
         self.exchanges = exchanges
-        self.codes = codes
+        if codes is None and code is not None:
+            self.codes = [code]
+        else:
+            self.codes = codes
         self.day_data = day_data
 
         # set entity_ids or (entity_type,exchanges,codes)
@@ -144,11 +156,22 @@ class EntityEventRecorder(Recorder):
 class TimeSeriesDataRecorder(EntityEventRecorder):
     default_size = 2000
 
-    def __init__(self, force_update=False, sleeping_time=5, exchanges=None, entity_ids=None, codes=None, day_data=False,
-                 entity_filters=None, ignore_failed=True, real_time=False, fix_duplicate_way='add',
-                 start_timestamp=None, end_timestamp=None) -> None:
-        super().__init__(force_update, sleeping_time, exchanges, entity_ids, codes, day_data, entity_filters,
-                         ignore_failed)
+    def __init__(self,
+                 force_update=False,
+                 sleeping_time=5,
+                 exchanges=None,
+                 entity_ids=None,
+                 code=None,
+                 codes=None,
+                 day_data=False,
+                 entity_filters=None,
+                 ignore_failed=True,
+                 real_time=False,
+                 fix_duplicate_way='add',
+                 start_timestamp=None,
+                 end_timestamp=None) -> None:
+        super().__init__(force_update, sleeping_time, exchanges, entity_ids, code=code, codes=codes, day_data=day_data,
+                         entity_filters=entity_filters, ignore_failed=ignore_failed)
 
         self.real_time = real_time
         self.close_hour, self.close_minute = self.entity_schema.get_close_hour_and_minute()
@@ -469,13 +492,26 @@ class TimeSeriesDataRecorder(EntityEventRecorder):
 
 
 class FixedCycleDataRecorder(TimeSeriesDataRecorder):
-    def __init__(self, force_update=True, sleeping_time=10, exchanges=None, entity_ids=None, codes=None, day_data=False,
-                 entity_filters=None, ignore_failed=True, real_time=False, fix_duplicate_way='ignore',
-                 start_timestamp=None, end_timestamp=None, level=IntervalLevel.LEVEL_1DAY, kdata_use_begin_time=False,
+    def __init__(self,
+                 force_update=True,
+                 sleeping_time=10,
+                 exchanges=None,
+                 entity_ids=None,
+                 code=None,
+                 codes=None,
+                 day_data=False,
+                 entity_filters=None,
+                 ignore_failed=True,
+                 real_time=False,
+                 fix_duplicate_way='ignore',
+                 start_timestamp=None,
+                 end_timestamp=None,
+                 level=IntervalLevel.LEVEL_1DAY,
+                 kdata_use_begin_time=False,
                  one_day_trading_minutes=24 * 60) -> None:
-        super().__init__(force_update, sleeping_time, exchanges, entity_ids, codes, day_data, entity_filters,
-                         ignore_failed, real_time=real_time, fix_duplicate_way=fix_duplicate_way,
-                         start_timestamp=start_timestamp,
+        super().__init__(force_update, sleeping_time, exchanges, entity_ids, code=code, codes=codes, day_data=day_data,
+                         entity_filters=entity_filters, ignore_failed=ignore_failed, real_time=real_time,
+                         fix_duplicate_way=fix_duplicate_way, start_timestamp=start_timestamp,
                          end_timestamp=end_timestamp)
 
         self.level = IntervalLevel(level)
@@ -535,12 +571,24 @@ class FixedCycleDataRecorder(TimeSeriesDataRecorder):
 
 class TimestampsDataRecorder(TimeSeriesDataRecorder):
 
-    def __init__(self, force_update=False, sleeping_time=5, exchanges=None, entity_ids=None, codes=None, day_data=False,
-                 entity_filters=None, ignore_failed=True, real_time=False, fix_duplicate_way='add',
-                 start_timestamp=None, end_timestamp=None) -> None:
-        super().__init__(force_update, sleeping_time, exchanges, entity_ids, codes, day_data, entity_filters,
-                         ignore_failed, real_time=real_time, fix_duplicate_way=fix_duplicate_way,
-                         start_timestamp=start_timestamp, end_timestamp=end_timestamp)
+    def __init__(self,
+                 force_update=False,
+                 sleeping_time=5,
+                 exchanges=None,
+                 entity_ids=None,
+                 code=None,
+                 codes=None,
+                 day_data=False,
+                 entity_filters=None,
+                 ignore_failed=True,
+                 real_time=False,
+                 fix_duplicate_way='add',
+                 start_timestamp=None,
+                 end_timestamp=None) -> None:
+        super().__init__(force_update, sleeping_time, exchanges, entity_ids, code=code, codes=codes, day_data=day_data,
+                         entity_filters=entity_filters, ignore_failed=ignore_failed, real_time=real_time,
+                         fix_duplicate_way=fix_duplicate_way, start_timestamp=start_timestamp,
+                         end_timestamp=end_timestamp)
         self.security_timestamps_map = {}
 
     def init_timestamps(self, entity_item) -> List[pd.Timestamp]:
@@ -579,5 +627,6 @@ class TimestampsDataRecorder(TimeSeriesDataRecorder):
         return timestamps[0], timestamps[-1], len(timestamps), timestamps
 
 
-__all__ = ['Recorder', 'EntityEventRecorder', 'FixedCycleDataRecorder', 'TimestampsDataRecorder',
-           'TimeSeriesDataRecorder']
+# the __all__ is generated
+__all__ = ['Meta', 'Recorder', 'EntityEventRecorder', 'TimeSeriesDataRecorder', 'FixedCycleDataRecorder',
+           'TimestampsDataRecorder']
