@@ -18,6 +18,16 @@ class RecorderState(ZvtInfoBase, Mixin):
     state = Column(Text())
 
 
+# 用于保存recorder的状态
+class TaggerState(ZvtInfoBase, Mixin):
+    __tablename__ = 'tagger_state'
+    # tagger名字
+    tagger_name = Column(String(length=128))
+
+    # json string
+    state = Column(Text())
+
+
 register_schema(providers=['zvt'], db_name='zvt_info', schema_base=ZvtInfoBase)
 # the __all__ is generated
-__all__ = ['RecorderState']
\ No newline at end of file
+__all__ = ['RecorderState', 'TaggerState']
