@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import declarative_base
 
 from zvt.contract import Mixin
 from zvt.contract.register import register_schema
@@ -13,6 +13,5 @@ class StockTradeDay(TradeDayBase, Mixin):
 
 register_schema(providers=['joinquant'], db_name='trade_day', schema_base=TradeDayBase)
 
-__all__ = ['StockTradeDay']
 # the __all__ is generated
 __all__ = ['StockTradeDay']
\ No newline at end of file
