@@ -11,6 +11,7 @@ class Stock1wkZenFactor(Stock1wkZenFactorBase, ZenFactorCommon):
     __tablename__ = 'stock_1wk_zen_factor'
 
 
-register_schema(providers=['zvt'], db_name='stock_1wk_zen_factor', schema_base=Stock1wkZenFactorBase)
+register_schema(providers=['zvt'], db_name='stock_1wk_zen_factor', schema_base=Stock1wkZenFactorBase,
+                entity_type='stock')
 # the __all__ is generated
-__all__ = ['Stock1wkZenFactor']
+__all__ = ['Stock1wkZenFactor']
\ No newline at end of file
