@@ -15,10 +15,9 @@ from zvt.contract.data_type import Bean
 from zvt.contract.drawer import Rect
 from zvt.contract.factor import Accumulator
 from zvt.contract.factor import Transformer
-from zvt.contract.normal_data import NormalData
-from zvt.domain import Stock
+from zvt.domain import Stock, Block
+from zvt.factors.algorithm import intersect, combine
 from zvt.factors.technical_factor import TechnicalFactor
-from zvt.factors.algorithm import intersect
 from zvt.utils import pd_is_not_null, to_string
 from zvt.utils import to_time_str
 from zvt.utils.time_utils import TIME_FORMAT_ISO8601
@@ -274,15 +273,23 @@ class ZenState(Bean):
         self.pre_bi = state.get('pre_bi')
         self.pre_duan = state.get('pre_duan')
 
+        # 目前的merge_zhongshu
+        self.merge_zhongshu = state.get('merge_zhongshu')
+        self.merge_zhongshu_level = state.get('merge_zhongshu_level')
+        self.merge_zhongshu_interval = state.get('merge_zhongshu_interval')
+
 
 def handle_zhongshu(points: list, acc_df, end_index, zhongshu_col='zhongshu', zhongshu_change_col='zhongshu_change'):
     zhongshu = None
     zhongshu_change = None
+    interval = None
 
     if len(points) == 4:
         x1 = points[0][0]
         x2 = points[3][0]
 
+        interval = points[3][2] - points[0][2]
+
         if points[0][1] < points[1][1]:
             # 向下段
             range = intersect((points[0][1], points[1][1]),
@@ -312,7 +319,7 @@ def handle_zhongshu(points: list, acc_df, end_index, zhongshu_col='zhongshu', zh
                 points = points[-1:]
             else:
                 points = points[1:]
-    return points, zhongshu, zhongshu_change
+    return points, zhongshu, zhongshu_change, interval
 
 
 class ZenAccumulator(Accumulator):
@@ -398,6 +405,13 @@ class ZenAccumulator(Accumulator):
             acc_df['current_zhongshu_change'] = np.NAN
             acc_df['current_zhongshu_y0'] = np.NAN
             acc_df['current_zhongshu_y1'] = np.NAN
+
+            acc_df['current_merge_zhongshu_change'] = np.NAN
+            acc_df['current_merge_zhongshu_y0'] = np.NAN
+            acc_df['current_merge_zhongshu_y1'] = np.NAN
+            acc_df['current_merge_zhongshu_level'] = np.NAN
+            acc_df['current_merge_zhongshu_interval'] = np.NAN
+
             # 目前走势的临时方向 其跟direction的的关系 确定了下一个分型
             acc_df['tmp_direction'] = None
             acc_df['opposite_change'] = np.NAN
@@ -420,16 +434,20 @@ class ZenAccumulator(Accumulator):
             acc_df['duan_interval'] = np.NAN
 
             # 记录在确定中枢的最后一个段的终点x1，值为Rect(x0,y0,x1,y1)
-            acc_df['zhongshu'] = np.NAN
+            acc_df['zhongshu'] = None
             acc_df['zhongshu_change'] = np.NAN
 
-            acc_df['bi_zhongshu'] = np.NAN
+            acc_df['bi_zhongshu'] = None
             acc_df['bi_zhongshu_change'] = np.NAN
 
+            acc_df['merge_zhongshu'] = None
+            acc_df['merge_zhongshu_change'] = np.NAN
+
             acc_df = acc_df.reset_index(drop=True)
 
             zen_state = ZenState(dict(fenxing_list=[], direction=None, can_fenxing=None, can_fenxing_index=None,
-                                      opposite_count=0, current_duan_state='yi', duans=[], pre_bi=None, pre_duan=None))
+                                      opposite_count=0, current_duan_state='yi', duans=[], pre_bi=None, pre_duan=None,
+                                      merge_zhongshu=None))
 
             zen_state.fenxing_list: List[Fenxing] = []
 
@@ -452,6 +470,10 @@ class ZenAccumulator(Accumulator):
         pre_index = start_index - 1
 
         tmp_direction = zen_state.direction
+        current_merge_zhongshu = decode_rect(zen_state.merge_zhongshu) if zen_state.merge_zhongshu else None
+        current_merge_zhongshu_change = None
+        current_merge_zhongshu_interval = zen_state.merge_zhongshu_interval
+        current_merge_zhongshu_level = zen_state.merge_zhongshu_level
 
         current_zhongshu = None
         current_zhongshu_change = None
@@ -484,6 +506,24 @@ class ZenAccumulator(Accumulator):
                 acc_df.loc[index, 'current_zhongshu_y1'] = acc_df.loc[index - 1, 'current_zhongshu_y1']
                 acc_df.loc[index, 'current_zhongshu_change'] = acc_df.loc[index - 1, 'current_zhongshu_change']
 
+            if current_merge_zhongshu:
+                # acc_df.loc[index, 'current_merge_zhongshu'] = current_merge_zhongshu
+                acc_df.loc[index, 'current_merge_zhongshu_y0'] = current_merge_zhongshu.y0
+                acc_df.loc[index, 'current_merge_zhongshu_y1'] = current_merge_zhongshu.y1
+                acc_df.loc[index, 'current_merge_zhongshu_change'] = current_merge_zhongshu_change
+                acc_df.loc[index, 'current_merge_zhongshu_level'] = current_merge_zhongshu_level
+                acc_df.loc[index, 'current_merge_zhongshu_interval'] = current_merge_zhongshu_interval
+            else:
+                # acc_df.loc[index, 'current_merge_zhongshu'] = acc_df.loc[index - 1, 'current_merge_zhongshu']
+                acc_df.loc[index, 'current_merge_zhongshu_y0'] = acc_df.loc[index - 1, 'current_merge_zhongshu_y0']
+                acc_df.loc[index, 'current_merge_zhongshu_y1'] = acc_df.loc[index - 1, 'current_merge_zhongshu_y1']
+                acc_df.loc[index, 'current_merge_zhongshu_change'] = acc_df.loc[
+                    index - 1, 'current_merge_zhongshu_change']
+                acc_df.loc[index, 'current_merge_zhongshu_level'] = acc_df.loc[
+                    index - 1, 'current_merge_zhongshu_level']
+                acc_df.loc[index, 'current_merge_zhongshu_interval'] = acc_df.loc[
+                    index - 1, 'current_merge_zhongshu_interval']
+
             # 处理包含关系
             handle_including(one_df=acc_df, index=index, kdata=kdata, pre_index=pre_index, pre_kdata=pre_kdata,
                              tmp_direction=tmp_direction)
@@ -563,14 +603,53 @@ class ZenAccumulator(Accumulator):
                             acc_df.loc[zen_state.can_fenxing_index, 'bi_interval'] = interval
 
                         # 记录用于计算笔中枢的笔
-                        zen_state.bis.append((acc_df.loc[zen_state.can_fenxing_index, 'timestamp'], bi_value))
+                        zen_state.bis.append((acc_df.loc[zen_state.can_fenxing_index, 'timestamp'], bi_value,
+                                              zen_state.can_fenxing_index))
 
-                        # 计算笔中枢
-                        zen_state.bis, current_zhongshu, current_zhongshu_change = handle_zhongshu(points=zen_state.bis,
-                                                                                                   acc_df=acc_df,
-                                                                                                   end_index=zen_state.can_fenxing_index,
-                                                                                                   zhongshu_col='bi_zhongshu',
-                                                                                                   zhongshu_change_col='bi_zhongshu_change')
+                        # 计算笔中枢，当下来说这个 中枢 是确定的，并且是不可变的
+                        # 但标记的点为 过去，注意在回测时最近的一个中枢可能用到未来函数，前一个才是 已知的
+                        # 所以记了一个 current_zhongshu_y0 current_zhongshu_y1 这个是可直接使用的
+                        end_index = zen_state.can_fenxing_index
+
+                        zen_state.bis, current_zhongshu, current_zhongshu_change, current_zhongshu_interval = handle_zhongshu(
+                            points=zen_state.bis,
+                            acc_df=acc_df,
+                            end_index=end_index,
+                            zhongshu_col='bi_zhongshu',
+                            zhongshu_change_col='bi_zhongshu_change')
+
+                        if not current_merge_zhongshu:
+                            current_merge_zhongshu = current_zhongshu
+                            current_merge_zhongshu_change = current_zhongshu_change
+                            current_merge_zhongshu_level = 1
+                            current_merge_zhongshu_interval = current_zhongshu_interval
+                        else:
+                            if current_zhongshu:
+                                range_a = (current_merge_zhongshu.y0, current_merge_zhongshu.y1)
+                                range_b = (current_zhongshu.y0, current_zhongshu.y1)
+                                combine_range = combine(range_a, range_b)
+                                if combine_range:
+                                    y0 = combine_range[0]
+                                    y1 = combine_range[1]
+                                    current_merge_zhongshu = Rect(x0=current_merge_zhongshu.x0, x1=current_zhongshu.x1,
+                                                                  y0=y0, y1=y1)
+                                    current_merge_zhongshu_change = abs(y0 - y1) / y0
+                                    current_merge_zhongshu_level = current_merge_zhongshu_level + 1
+                                    current_merge_zhongshu_interval = current_merge_zhongshu_interval + current_zhongshu_interval
+                                else:
+                                    current_merge_zhongshu = current_zhongshu
+                                    current_merge_zhongshu_change = current_zhongshu_change
+                                    current_merge_zhongshu_level = 1
+                                    current_merge_zhongshu_interval = current_zhongshu_interval
+
+                                acc_df.loc[end_index, 'merge_zhongshu'] = current_merge_zhongshu
+                                acc_df.loc[end_index, 'merge_zhongshu_change'] = current_merge_zhongshu_change
+                                acc_df.loc[end_index, 'merge_zhongshu_level'] = current_merge_zhongshu_level
+                                acc_df.loc[end_index, 'merge_zhongshu_interval'] = current_merge_zhongshu_interval
+
+                        zen_state.merge_zhongshu = current_merge_zhongshu
+                        zen_state.merge_zhongshu_interval = current_merge_zhongshu_interval
+                        zen_state.merge_zhongshu_level = current_merge_zhongshu_level
 
                         zen_state.pre_bi = (zen_state.can_fenxing_index, bi_value)
 
@@ -624,13 +703,14 @@ class ZenAccumulator(Accumulator):
                                     zen_state.pre_duan = (duan_index, duan_value)
 
                                     # 记录用于计算中枢的段
-                                    zen_state.duans.append((acc_df.loc[duan_index, 'timestamp'], duan_value))
+                                    zen_state.duans.append(
+                                        (acc_df.loc[duan_index, 'timestamp'], duan_value, duan_index))
 
                                     # 计算中枢
-                                    zen_state.duans, _, _ = handle_zhongshu(points=zen_state.duans, acc_df=acc_df,
-                                                                            end_index=duan_index,
-                                                                            zhongshu_col='zhongshu',
-                                                                            zhongshu_change_col='zhongshu_change')
+                                    zen_state.duans, _, _, _ = handle_zhongshu(points=zen_state.duans, acc_df=acc_df,
+                                                                               end_index=duan_index,
+                                                                               zhongshu_col='zhongshu',
+                                                                               zhongshu_change_col='zhongshu_change')
 
                                     # 只留最后一个
                                     zen_state.fenxing_list = zen_state.fenxing_list[-1:]
@@ -650,9 +730,6 @@ class ZenAccumulator(Accumulator):
 class ZenFactor(TechnicalFactor):
     accumulator = ZenAccumulator()
 
-    def drawer_main_data(self) -> Optional[NormalData]:
-        return super().drawer_main_data()
-
     def __init__(self,
                  entity_schema: Type[TradableEntity] = Stock,
                  provider: str = None,
@@ -690,8 +767,8 @@ class ZenFactor(TechnicalFactor):
     def factor_col_map_object_hook(self) -> dict:
         return {
             'zhongshu': decode_rect,
-            'current_zhongshu': decode_rect,
-            'bi_zhongshu': decode_rect
+            'bi_zhongshu': decode_rect,
+            'merge_zhongshu': decode_rect
         }
 
     def factor_encoder(self):
@@ -699,12 +776,12 @@ class ZenFactor(TechnicalFactor):
 
     def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
         bi_value = self.factor_df[['bi_value']].dropna()
-        duan_value = self.factor_df[['duan_value']].dropna()
-        return [bi_value, duan_value]
+        # duan_value = self.factor_df[['duan_value']].dropna()
+        return [bi_value]
 
     def drawer_rects(self) -> List[Rect]:
-        df1 = self.factor_df[['bi_zhongshu']].dropna()
-        return df1['bi_zhongshu'].tolist()
+        df1 = self.factor_df[['merge_zhongshu']].dropna()
+        return df1['merge_zhongshu'].tolist()
 
     def drawer_sub_df_list(self) -> Optional[List[pd.DataFrame]]:
         # bi_slope = self.factor_df[['bi_slope']].dropna()
@@ -712,189 +789,22 @@ class ZenFactor(TechnicalFactor):
         # power = self.factor_df[['fenxing_power']].dropna()
         # zhongshu_change = self.factor_df[['zhongshu_change']].dropna()
         # return [bi_slope, duan_slope, power, zhongshu_change]
-        change1 = self.factor_df[['current_change']].dropna()
-        change2 = self.factor_df[['opposite_change']].dropna()
-        return [change1, change2]
-
-    def drawer_annotation_df(self) -> Optional[pd.DataFrame]:
-        if pd_is_not_null(self.result_df):
-            annotation_df = self.result_df.copy()
-            annotation_df = annotation_df[annotation_df['score']]
-            annotation_df['value'] = self.factor_df.loc[annotation_df.index]['close']
-            annotation_df['flag'] = annotation_df['score'].apply(lambda x: order_type_flag(x))
-            return annotation_df
-
-
-def order_type_flag(order_type):
-    if order_type:
-        return 'B'
-
-
-class TrendingFactor(ZenFactor):
-    def __init__(self,
-                 entity_schema: Type[TradableEntity] = Stock,
-                 provider: str = None,
-                 entity_provider: str = None,
-                 entity_ids: List[str] = None,
-                 exchanges: List[str] = None,
-                 codes: List[str] = None,
-                 start_timestamp: Union[str, pd.Timestamp] = None,
-                 end_timestamp: Union[str, pd.Timestamp] = None,
-                 columns: List = None,
-                 filters: List = None,
-                 order: object = None,
-                 limit: int = None,
-                 level: Union[str, IntervalLevel] = IntervalLevel.LEVEL_1DAY,
-                 category_field: str = 'entity_id',
-                 time_field: str = 'timestamp',
-                 computing_window: int = None,
-                 keep_all_timestamp: bool = False,
-                 fill_method: str = 'ffill',
-                 effective_number: int = None,
-                 transformer: Transformer = None,
-                 accumulator: Accumulator = None,
-                 need_persist: bool = False,
-                 only_compute_factor: bool = False,
-                 factor_name: str = None,
-                 clear_state: bool = False,
-                 only_load_factor: bool = True,
-                 adjust_type: Union[AdjustType, str] = None) -> None:
-        super().__init__(entity_schema, provider, entity_provider, entity_ids, exchanges, codes, start_timestamp,
-                         end_timestamp, columns, filters, order, limit, level, category_field, time_field,
-                         computing_window, keep_all_timestamp, fill_method, effective_number, transformer, accumulator,
-                         need_persist, only_compute_factor, factor_name, clear_state, only_load_factor, adjust_type)
-
-    def compute_result(self):
-        super().compute_result()
-        # 窄幅震荡
-        s1 = self.factor_df['current_zhongshu_change'] <= 0.15
-        s2 = self.factor_df['close'] > self.factor_df['current_zhongshu_y1'] * 1.1
-        s = s1 & s2
-
-        self.result_df = s.to_frame(name='score')
-
-    def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
-        zhongshu_y0 = self.factor_df[['current_zhongshu_y0']].dropna()
-        zhongshu_y1 = self.factor_df[['current_zhongshu_y1']].dropna()
-        return [zhongshu_y0, zhongshu_y1]
-
-    def drawer_main_df(self) -> Optional[pd.DataFrame]:
-        return self.factor_df
-
-
-class ShakingFactor(ZenFactor):
-    # 震荡区间
-    shaking_range = 0.15
-
-    def __init__(self,
-                 entity_schema: Type[TradableEntity] = Stock,
-                 provider: str = None,
-                 entity_provider: str = None,
-                 entity_ids: List[str] = None,
-                 exchanges: List[str] = None,
-                 codes: List[str] = None,
-                 start_timestamp: Union[str, pd.Timestamp] = None,
-                 end_timestamp: Union[str, pd.Timestamp] = None,
-                 columns: List = None,
-                 filters: List = None,
-                 order: object = None,
-                 limit: int = None,
-                 level: Union[str, IntervalLevel] = IntervalLevel.LEVEL_1DAY,
-                 category_field: str = 'entity_id',
-                 time_field: str = 'timestamp',
-                 computing_window: int = None,
-                 keep_all_timestamp: bool = False,
-                 fill_method: str = 'ffill',
-                 effective_number: int = None,
-                 transformer: Transformer = None,
-                 accumulator: Accumulator = None,
-                 need_persist: bool = False,
-                 only_compute_factor: bool = False,
-                 factor_name: str = None,
-                 clear_state: bool = False,
-                 only_load_factor: bool = True,
-                 adjust_type: Union[AdjustType, str] = None) -> None:
-        super().__init__(entity_schema, provider, entity_provider, entity_ids, exchanges, codes, start_timestamp,
-                         end_timestamp, columns, filters, order, limit, level, category_field, time_field,
-                         computing_window, keep_all_timestamp, fill_method, effective_number, transformer, accumulator,
-                         need_persist, only_compute_factor, factor_name, clear_state, only_load_factor, adjust_type)
-
-    def compute_result(self):
-        super().compute_result()
-        s1 = self.factor_df['current_zhongshu_change'] >= self.shaking_range
-        # 震荡低点
-        middle = (self.factor_df['current_zhongshu_y0'] + self.factor_df['current_zhongshu_y1']) / 2
-        s2 = self.factor_df['close'] <= middle
-        s3 = self.factor_df['close'] >= 0.9 * self.factor_df['current_zhongshu_y0']
-        s = s1 & s2 & s3
-
-        self.result_df = s.to_frame(name='score')
-
-    def drawer_annotation_df(self) -> Optional[pd.DataFrame]:
-        if pd_is_not_null(self.result_df):
-            annotation_df = self.result_df.copy()
-            annotation_df = annotation_df[annotation_df['score']]
-            annotation_df['value'] = self.factor_df.loc[annotation_df.index]['close']
-            annotation_df['flag'] = annotation_df['score'].apply(lambda x: order_type_flag(x))
-            return annotation_df
-
-    def drawer_main_df(self) -> Optional[pd.DataFrame]:
-        return self.factor_df
-
-    def drawer_factor_df_list(self) -> Optional[List[pd.DataFrame]]:
-        zhongshu_y0 = self.factor_df[['current_zhongshu_y0']].dropna()
-        zhongshu_y1 = self.factor_df[['current_zhongshu_y1']].dropna()
-        return [zhongshu_y0, zhongshu_y1]
-
-class ShowFactor(ZenFactor):
-
-    def __init__(self,
-                 entity_schema: Type[TradableEntity] = Stock,
-                 provider: str = None,
-                 entity_provider: str = None,
-                 entity_ids: List[str] = None,
-                 exchanges: List[str] = None,
-                 codes: List[str] = None,
-                 start_timestamp: Union[str, pd.Timestamp] = None,
-                 end_timestamp: Union[str, pd.Timestamp] = None,
-                 columns: List = None,
-                 filters: List = None,
-                 order: object = None,
-                 limit: int = None,
-                 level: Union[str, IntervalLevel] = IntervalLevel.LEVEL_1DAY,
-                 category_field: str = 'entity_id',
-                 time_field: str = 'timestamp',
-                 computing_window: int = None,
-                 keep_all_timestamp: bool = False,
-                 fill_method: str = 'ffill',
-                 effective_number: int = None,
-                 transformer: Transformer = None,
-                 accumulator: Accumulator = None,
-                 need_persist: bool = False,
-                 only_compute_factor: bool = False,
-                 factor_name: str = None,
-                 clear_state: bool = False,
-                 only_load_factor: bool = True,
-                 adjust_type: Union[AdjustType, str] = None) -> None:
-        super().__init__(entity_schema, provider, entity_provider, entity_ids, exchanges, codes, start_timestamp,
-                         end_timestamp, columns, filters, order, limit, level, category_field, time_field,
-                         computing_window, keep_all_timestamp, fill_method, effective_number, transformer, accumulator,
-                         need_persist, only_compute_factor, factor_name, clear_state, only_load_factor, adjust_type)
-
-    def drawer_sub_df_list(self) -> Optional[List[pd.DataFrame]]:
-        change1 = self.factor_df[['current_slope']].dropna()
-        change2 = self.factor_df[['opposite_slope']].dropna()
-        return [change1, change2]
+        change1 = self.factor_df[['current_merge_zhongshu_level']].dropna()
+        # change2 = self.factor_df[['opposite_change']].dropna()
+        return [change1]
 
 
 if __name__ == '__main__':
-    from zvt.factors.zen.domain import Stock1dZenFactor
-    from zvt.domain import Stock1dHfqKdata
+    entity_ids = ['block_cn_BK0429']
+    # Stock1dHfqKdata.record_data(entity_ids=entity_ids)
 
-    Stock1dHfqKdata.record_data(codes=['000338','601318'])
-    zen = ZenFactor(level='1d', need_persist=True, clear_state=False, codes=['000338','601318'])
-    zen.draw(show=True)
+    f = ZenFactor(entity_schema=Block, entity_ids=entity_ids, need_persist=False, provider='em',
+                  entity_provider='eastmoney')
+    f.draw(show=True)
 
 # the __all__ is generated
-__all__ = ['Direction', 'Fenxing', 'KState', 'DuanState', 'fenxing_power', 'a_include_b', 'is_including', 'get_direction', 'is_up', 'is_down', 'handle_first_fenxing', 'handle_duan', 'handle_including', 'FactorStateEncoder', 'decode_rect', 'decode_fenxing',
-           'get_zen_factor_schema', 'ZenState', 'handle_zhongshu', 'ZenAccumulator', 'ZenFactor', 'order_type_flag', 'TrendingFactor', 'ShakingFactor']
\ No newline at end of file
+__all__ = ['Direction', 'Fenxing', 'KState', 'DuanState', 'fenxing_power', 'a_include_b', 'is_including',
+           'get_direction', 'is_up', 'is_down', 'handle_first_fenxing', 'handle_duan', 'handle_including',
+           'FactorStateEncoder', 'decode_rect', 'decode_fenxing',
+           'get_zen_factor_schema', 'ZenState', 'handle_zhongshu', 'ZenAccumulator', 'ZenFactor',
+           ]
